@@ -14,4 +14,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
-__version__ = '0.6.0'
+from .pie import Pie
+from . import exceptions
+
+__version__ = '1.0.1'
