@@ -14,6 +14,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
+from functools import wraps
 import hashlib
 import json
 import os
@@ -28,6 +29,17 @@ from . import exceptions
 from .ignore import get_not_ignored_files
 
 
+def repo_required(method):
+    @wraps(method)
+    def decorator(ref, *args, **kwargs):
+        if not ref.check_repo_files():
+            raise exceptions.RepositoryNotExistsError('Repository not exists')
+        else:
+            return method(ref, *args, **kwargs)
+    
+    return decorator
+
+
 class Pie(object):
     def __init__(self) -> None:
         """Creating a new instance from Pie."""
@@ -38,18 +50,39 @@ class Pie(object):
         self.pieces_file = os.path.join(self.repository_path, 'pieces.json')
         self.pieces_dir = os.path.join(self.repository_path, 'pieces')
 
+    def check_repo_files(self) -> bool:
+        """Checks all files in the repository
+        and returns `True` if all are present.
+
+        :return: True if all are present.
+        :rtype: bool
+        """
+
+        files = [
+            os.path.isdir(self.repository_path),
+            os.path.isfile(self.repository_info_file),
+            os.path.isfile(self.author_info_file),
+            os.path.isdir(self.pieces_dir),
+            os.path.isfile(self.pieces_file),
+        ]
+
+        return all(files)
+
+    @repo_required
     def _get_repo_info(self) -> dict:
         with open(self.repository_info_file, 'r') as reader:
             repo_info = json.load(reader)
 
         return repo_info
 
+    @repo_required
     def _get_pieces_refs(self) -> dict:
         with open(self.pieces_file, 'r') as reader:
             pieces_refs = json.load(reader)
 
         return pieces_refs
 
+    @repo_required
     def _get_author_info(self) -> dict:
         with open(self.author_info_file, 'r') as reader:
             author_info = json.load(reader)
@@ -110,6 +143,7 @@ class Pie(object):
         self._write_pieces_refs(pieces_object)
         self._write_author_info(author, author_email)
 
+    @repo_required
     def get_tracked_files(self) -> dict:
         """Get the tracked files
 
@@ -121,6 +155,7 @@ class Pie(object):
         pieces_refs = self._get_pieces_refs()
         return pieces_refs['tracked']
 
+    @repo_required
     def track_file(self, filepath: str) -> None:
         """Adds a new file to the trace.
 
@@ -222,9 +257,9 @@ class Pie(object):
 
     def _decode_piece_token(self, piece_token: str) -> dict:
         integrity_exceptions = (
-                utoken.exceptions.InvalidContentTokenError,
-                utoken.exceptions.InvalidKeyError,
-                utoken.exceptions.InvalidTokenError
+            utoken.exceptions.InvalidContentTokenError,
+            utoken.exceptions.InvalidKeyError,
+            utoken.exceptions.InvalidTokenError
         )
 
         try:
@@ -234,6 +269,7 @@ class Pie(object):
 
         return piece_info
 
+    @repo_required
     def join_file_changes(self, filepath: str) -> dict:
         """Merge all committed changes from the
         file into a single dictionary with numbered lines.
@@ -275,20 +311,26 @@ class Pie(object):
 
         return previous_lines
 
-    def _create_commit(self, files_refs: dict, message: str) -> dict:
+    @repo_required
+    def _create_commit(self, files_refs: dict, message: str) -> Union[dict, str]:
         author_info = self._get_author_info()
         pieces_refs = self._get_pieces_refs()
 
+        files = {}
+
+        for file, info in files_refs.items():
+            files[file] = self._create_piece(info['previous_hash'], info['diff'])
+
         commit = {
             'author': author_info['author'],
             'author_email': author_info['author_email'],
             'message': message,
             'datetime': datetime.now().strftime('%Y.%m.%d %H:%M:%S'),
-            'files': files_refs
+            'files': files
         }
 
         commit_json = json.dumps(commit).encode()
-        commit_hash = hashlib.sha256(commit_json).hexdigest()
+        commit_hash = hashlib.sha1(commit_json).hexdigest()
 
         for file in files_refs:
             pieces_refs['tracked'][file]['commits'].append(commit_hash)
@@ -296,8 +338,9 @@ class Pie(object):
         pieces_refs['commits'][commit_hash] = commit
         self._write_pieces_refs(pieces_refs)
 
-        return commit
+        return commit, commit_hash
 
+    @repo_required
     def _create_piece(self, previous_hash: str, lines: dict) -> str:
         repo_info = self._get_repo_info()
 
@@ -316,6 +359,7 @@ class Pie(object):
 
         return piece_id
 
+    @repo_required
     def _get_last_piece_hash(self, filepath: str) -> str:
         pieces_refs = self._get_pieces_refs()
         file = pieces_refs['tracked'][filepath]
@@ -335,6 +379,7 @@ class Pie(object):
 
         return hashlib.sha256(piece_json).hexdigest()
 
+    @repo_required
     def get_commits(self) -> dict:
         """Return all commits.
 
@@ -357,6 +402,7 @@ class Pie(object):
 
         return previous_hash != current_hash
 
+    @repo_required
     def get_files_status(self) -> List[dict]:
         """Returns the status of new, uncommitted and
         untracked files (as long as they are not in the .ignore file).
@@ -396,7 +442,8 @@ class Pie(object):
 
         return files_status
 
-    def commit(self, filepath_list: list, message: str) -> dict:
+    @repo_required
+    def commit(self, filepath_list: list, message: str) -> Union[dict, str]:
         """Commit the files.
 
         :param filepath_list: List of files that have changed.
@@ -404,7 +451,7 @@ class Pie(object):
         :param message: Commit message.
         :type message: str
         :return: Return the commit data.
-        :rtype: dict
+        :rtype: Union[dict, str]
         """
 
         tracked_files = self.get_tracked_files()
@@ -418,7 +465,10 @@ class Pie(object):
 
             if file_info:
                 if not file_info['commits']:
-                    file_refs[filepath] = self._create_piece(0, self.index_file_lines(filepath))
+                    file_refs[filepath] = {
+                        'diff': self.index_file_lines(filepath),
+                        'previous_hash': 0
+                    }
                 else:
                     with futures.ThreadPoolExecutor() as executor:
                         th1 = executor.submit(self._get_last_piece_hash, filepath)
@@ -430,7 +480,17 @@ class Pie(object):
                         current_lines = th3.result()
 
                     lines_difference = self.get_lines_difference(previous_lines, current_lines)
-                    file_refs[filepath] = self._create_piece(previous_hash, lines_difference)
+
+                    if lines_difference:
+                        file_refs[filepath] = {
+                            'diff': lines_difference,
+                            'previous_hash': previous_hash
+                        }
+            else:
+                raise exceptions.FileNotTrackedError(f'File "{filepath}" not tracked')
+
+        if not file_refs:
+            raise exceptions.NoFilesToCommitError('No files to commit')
 
         return self._create_commit(file_refs, message)
 
@@ -447,6 +507,7 @@ class Pie(object):
         else:
             return False
 
+    @repo_required
     def merge(self) -> list:
         """Merge all committed files.
 
